#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const inquirer_1 = __importDefault(require("inquirer"));
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = __importDefault(require("fs"));
const ora_1 = __importDefault(require("ora"));
const download_git_repo_1 = __importDefault(require("download-git-repo"));
const log_symbols_1 = __importDefault(require("log-symbols"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const utils_1 = require("./utils");
const child_process_1 = require("child_process");
const tips = (param) => {
    const { install, installMode = 'pnpm', projectName } = param || {};
    console.log(log_symbols_1.default.success, chalk_1.default.green('The project is initialized'));
    console.log('\n To get started');
    console.log(`\n cd ${projectName}`);
    !install && console.log(`\n ${installMode} install`);
    console.log(`\n ${installMode === 'yarn' ? 'yarn dev' : `${installMode} run dev`} `);
};
const has = (0, utils_1.hasPackage)();
const iMode = () => {
    if (has('pnpm'))
        return 'pnpm';
    if (has('yarn'))
        return 'yarn';
    return 'npm';
};
const create = (projectName, options) => __awaiter(void 0, void 0, void 0, function* () {
    if (!has('git')) {
        console.log(chalk_1.default.red("Error, Git is not exsits ,please install git!"));
        return;
    }
    if (has('node')) {
        let nodeVersion = (0, child_process_1.execSync)(`node --version`).toString();
        nodeVersion = nodeVersion.substring(1, nodeVersion.length);
        nodeVersion = parseInt(nodeVersion.split('.').join('').substring(0, 4));
        if (nodeVersion < 1418) {
            console.log(chalk_1.default.red("Error, node version must be at least v14.18.0!"));
            return;
        }
    }
    const { force } = options || {};
    if (!force && fs_1.default.existsSync(projectName)) {
        console.log(chalk_1.default.red("Error, In this directory, the project name already exsits !"));
        console.log(chalk_1.default.green('You can use option -f to force delete the directory except for direct mode !'));
        return;
    }
    const template = Promise.resolve().then(() => __importStar(require((0, utils_1.resolve)('../template'))));
    const templateObj = yield template;
    const choices = [];
    Object.keys(templateObj.default).forEach(key => {
        choices.push(`${key} (${templateObj.default[key].description})`);
    });
    const param = [
        {
            type: 'list',
            message: '请选择一个预设',
            name: 'templateName',
            choices: choices,
            filter: (val) => val.split(' ')[0]
        },
        {
            type: 'input',
            message: '请输入项目描述',
            name: 'description',
        },
        {
            type: 'input',
            message: '请输入作者名称',
            name: 'author',
        },
        {
            type: 'confirm',
            message: '请确认是否安装npm包?',
            name: 'install',
        },
        {
            type: 'list',
            message: '请选择一个安装方式',
            name: 'installMode',
            default: iMode(),
            choices: ['pnpm', 'npm', 'yarn'],
            when: (answers) => {
                const { install, templateName } = answers;
                return install && templateName !== 'monorope-component';
            }
        },
    ];
    const answers = yield inquirer_1.default.prompt(param);
    if (fs_1.default.existsSync(projectName)) {
        console.log(chalk_1.default.white('\n Start Delete original item... \n'));
        const spinner = (0, ora_1.default)("Delete...");
        spinner.start();
        yield fs_extra_1.default.remove(projectName);
        spinner.succeed();
    }
    const { templateName, description, author, install, installMode = 'pnpm' } = answers || {};
    const url = templateObj[templateName].url;
    const direct = url.split(':')[0] === 'direct';
    console.log(chalk_1.default.white('\n Start generating... \n'));
    const spinner = (0, ora_1.default)("Downloading...");
    spinner.start();
    (0, download_git_repo_1.default)(url, projectName, { clone: direct }, (err) => __awaiter(void 0, void 0, void 0, function* () {
        if (err) {
            spinner.fail();
            console.log(chalk_1.default.red(`Generation failed. ${err}`));
            return;
        }
        const meta = {
            description: description,
            name: projectName.replace(/\/|\\/g, '-'),
            author: author
        };
        const filePath = `${projectName}/package.json`;
        const packageJsonData = (0, utils_1.readFile)(filePath);
        Object.assign(packageJsonData, meta);
        fs_1.default.writeFileSync(filePath, JSON.stringify(packageJsonData, null, 2));
        spinner.succeed();
        const gitignorePath = `${projectName}/.gitignore`;
        const fileContent = (0, utils_1.readFile)(`${projectName}/.gitignore`);
        const newContent = fileContent
            .split('\n')
            .filter(line => !line.includes('pnpm-lock.yaml'))
            .join('\n');
        fs_1.default.writeFileSync(gitignorePath, newContent, 'utf-8');
        try {
            console.log(chalk_1.default.white('\n Start update... \n'));
            const output = yield (0, child_process_1.execSync)(`npx  npm-check-updates@16.14.20 -u --target semver  --packageManager ${installMode}`, {
                cwd: projectName,
                encoding: 'utf-8',
                stdio: 'inherit'
            });
            output && console.log('Command output:\n', output);
        }
        catch (error) {
            console.error('Error executing command:\n', error.message);
        }
        if (install) {
            const stdout = yield (0, utils_1.shellExec)({
                directive: `${installMode} install --force`,
                cwd: projectName,
                loading: 'Install...',
                log: `${installMode} install`
            });
            console.log(stdout);
            tips({ projectName, install, installMode });
            return;
        }
        tips({ projectName, install, installMode });
    }));
});
exports.create = create;
